@@ -12,7 +12,6 @@ import operator
 import os
 import os.path
 import textwrap
-
 from collections import defaultdict
 from inspect import signature
 from itertools import islice
@@ -20,23 +19,40 @@ from itertools import islice
 import numpy as np
 
 from astropy import __version__
-
-from .card import Card, BLANK_CARD
-from .header import Header
-# HDUList is used in one of the doctests
-from .hdu.hdulist import fitsopen, HDUList  # pylint: disable=W0611
-from .hdu.table import _TableLikeHDU
-from astropy.utils.diff import (report_diff_values, fixed_width_indent,
-                                where_not_allclose, diff_values)
+from astropy.utils.diff import (
+    diff_values,
+    fixed_width_indent,
+    report_diff_values,
+    where_not_allclose,
+)
 from astropy.utils.misc import NOT_OVERWRITING_MSG
 
-__all__ = ['FITSDiff', 'HDUDiff', 'HeaderDiff', 'ImageDataDiff', 'RawDataDiff',
-           'TableDataDiff']
+from .card import BLANK_CARD, Card
+
+# HDUList is used in one of the doctests
+from .hdu.hdulist import HDUList, fitsopen  # pylint: disable=W0611
+from .hdu.table import _TableLikeHDU
+from .header import Header
+from .util import path_like
+
+__all__ = [
+    "FITSDiff",
+    "HDUDiff",
+    "HeaderDiff",
+    "ImageDataDiff",
+    "RawDataDiff",
+    "TableDataDiff",
+]
 
 # Column attributes of interest for comparison
-_COL_ATTRS = [('unit', 'units'), ('null', 'null values'),
-              ('bscale', 'bscales'), ('bzero', 'bzeros'),
-              ('disp', 'display formats'), ('dim', 'dimensions')]
+_COL_ATTRS = [
+    ("unit", "units"),
+    ("null", "null values"),
+    ("bscale", "bscales"),
+    ("bzero", "bzeros"),
+    ("disp", "display formats"),
+    ("dim", "dimensions"),
+]
 
 
 class _BaseDiff:
@@ -117,8 +133,9 @@ class _BaseDiff:
         difference was found between the two objects being compared.
         """
 
-        return not any(getattr(self, attr) for attr in self.__dict__
-                       if attr.startswith('diff_'))
+        return not any(
+            getattr(self, attr) for attr in self.__dict__ if attr.startswith("diff_")
+        )
 
     def report(self, fileobj=None, indent=0, overwrite=False):
         """
@@ -150,12 +167,13 @@ class _BaseDiff:
         return_string = False
         filepath = None
 
-        if isinstance(fileobj, str):
+        if isinstance(fileobj, path_like):
+            fileobj = os.path.expanduser(fileobj)
             if os.path.exists(fileobj) and not overwrite:
                 raise OSError(NOT_OVERWRITING_MSG.format(fileobj))
             else:
                 filepath = fileobj
-                fileobj = open(filepath, 'w')
+                fileobj = open(filepath, "w")
         elif fileobj is None:
             fileobj = io.StringIO()
             return_string = True
@@ -173,7 +191,7 @@ class _BaseDiff:
             return fileobj.getvalue()
 
     def _writeln(self, text):
-        self._fileobj.write(fixed_width_indent(text, self._indent) + '\n')
+        self._fileobj.write(fixed_width_indent(text, self._indent) + "\n")
 
     def _diff(self):
         raise NotImplementedError
@@ -196,10 +214,20 @@ class FITSDiff(_BaseDiff):
       representing the differences between the two HDUs.
     """
 
-    def __init__(self, a, b, ignore_hdus=[], ignore_keywords=[],
-                 ignore_comments=[], ignore_fields=[],
-                 numdiffs=10, rtol=0.0, atol=0.0,
-                 ignore_blanks=True, ignore_blank_cards=True):
+    def __init__(
+        self,
+        a,
+        b,
+        ignore_hdus=[],
+        ignore_keywords=[],
+        ignore_comments=[],
+        ignore_fields=[],
+        numdiffs=10,
+        rtol=0.0,
+        atol=0.0,
+        ignore_blanks=True,
+        ignore_blank_cards=True,
+    ):
         """
         Parameters
         ----------
@@ -268,8 +296,11 @@ class FITSDiff(_BaseDiff):
             try:
                 a = fitsopen(a)
             except Exception as exc:
-                raise OSError("error opening file a ({}): {}: {}".format(
-                        a, exc.__class__.__name__, exc.args[0]))
+                raise OSError(
+                    "error opening file a ({}): {}: {}".format(
+                        a, exc.__class__.__name__, exc.args[0]
+                    )
+                )
             close_a = True
         else:
             close_a = False
@@ -278,17 +309,20 @@ class FITSDiff(_BaseDiff):
             try:
                 b = fitsopen(b)
             except Exception as exc:
-                raise OSError("error opening file b ({}): {}: {}".format(
-                        b, exc.__class__.__name__, exc.args[0]))
+                raise OSError(
+                    "error opening file b ({}): {}: {}".format(
+                        b, exc.__class__.__name__, exc.args[0]
+                    )
+                )
             close_b = True
         else:
             close_b = False
 
         # Normalize keywords/fields to ignore to upper case
-        self.ignore_hdus = set(k.upper() for k in ignore_hdus)
-        self.ignore_keywords = set(k.upper() for k in ignore_keywords)
-        self.ignore_comments = set(k.upper() for k in ignore_comments)
-        self.ignore_fields = set(k.upper() for k in ignore_fields)
+        self.ignore_hdus = {k.upper() for k in ignore_hdus}
+        self.ignore_keywords = {k.upper() for k in ignore_keywords}
+        self.ignore_comments = {k.upper() for k in ignore_comments}
+        self.ignore_fields = {k.upper() for k in ignore_fields}
 
         self.numdiffs = numdiffs
         self.rtol = rtol
@@ -300,7 +334,7 @@ class FITSDiff(_BaseDiff):
         # Some hdu names may be pattern wildcards.  Find them.
         self.ignore_hdu_patterns = set()
         for name in list(self.ignore_hdus):
-            if name != '*' and glob.has_magic(name):
+            if name != "*" and glob.has_magic(name):
                 self.ignore_hdus.remove(name)
                 self.ignore_hdu_patterns.add(name)
 
@@ -322,11 +356,11 @@ class FITSDiff(_BaseDiff):
         # Record filenames for use later in _report
         self.filenamea = self.a.filename()
         if not self.filenamea:
-            self.filenamea = f'<{self.a.__class__.__name__} object at {id(self.a):#x}>'
+            self.filenamea = f"<{self.a.__class__.__name__} object at {id(self.a):#x}>"
 
         self.filenameb = self.b.filename()
         if not self.filenameb:
-            self.filenameb = f'<{self.b.__class__.__name__} object at {id(self.b):#x}>'
+            self.filenameb = f"<{self.b.__class__.__name__} object at {id(self.b):#x}>"
 
         if self.ignore_hdus:
             self.a = HDUList([h for h in self.a if h.name not in self.ignore_hdus])
@@ -335,10 +369,20 @@ class FITSDiff(_BaseDiff):
             a_names = [hdu.name for hdu in self.a]
             b_names = [hdu.name for hdu in self.b]
             for pattern in self.ignore_hdu_patterns:
-                self.a = HDUList([h for h in self.a if h.name not in fnmatch.filter(
-                    a_names, pattern)])
-                self.b = HDUList([h for h in self.b if h.name not in fnmatch.filter(
-                    b_names, pattern)])
+                self.a = HDUList(
+                    [
+                        h
+                        for h in self.a
+                        if h.name not in fnmatch.filter(a_names, pattern)
+                    ]
+                )
+                self.b = HDUList(
+                    [
+                        h
+                        for h in self.b
+                        if h.name not in fnmatch.filter(b_names, pattern)
+                    ]
+                )
 
         # For now, just compare the extensions one by one in order.
         # Might allow some more sophisticated types of diffing later.
@@ -349,73 +393,95 @@ class FITSDiff(_BaseDiff):
             hdu_diff = HDUDiff.fromdiff(self, self.a[idx], self.b[idx])
 
             if not hdu_diff.identical:
-                if self.a[idx].name == self.b[idx].name and self.a[idx].ver == self.b[idx].ver:
-                    self.diff_hdus.append((idx, hdu_diff, self.a[idx].name, self.a[idx].ver))
+                if (
+                    self.a[idx].name == self.b[idx].name
+                    and self.a[idx].ver == self.b[idx].ver
+                ):
+                    self.diff_hdus.append(
+                        (idx, hdu_diff, self.a[idx].name, self.a[idx].ver)
+                    )
                 else:
                     self.diff_hdus.append((idx, hdu_diff, "", self.a[idx].ver))
 
     def _report(self):
-        wrapper = textwrap.TextWrapper(initial_indent='  ',
-                                       subsequent_indent='  ')
+        wrapper = textwrap.TextWrapper(initial_indent="  ", subsequent_indent="  ")
 
-        self._fileobj.write('\n')
-        self._writeln(f' fitsdiff: {__version__}')
-        self._writeln(f' a: {self.filenamea}\n b: {self.filenameb}')
+        self._fileobj.write("\n")
+        self._writeln(f" fitsdiff: {__version__}")
+        self._writeln(f" a: {self.filenamea}\n b: {self.filenameb}")
 
         if self.ignore_hdus:
-            ignore_hdus = ' '.join(sorted(self.ignore_hdus))
-            self._writeln(f' HDU(s) not to be compared:\n{wrapper.fill(ignore_hdus)}')
+            ignore_hdus = " ".join(sorted(self.ignore_hdus))
+            self._writeln(f" HDU(s) not to be compared:\n{wrapper.fill(ignore_hdus)}")
 
         if self.ignore_hdu_patterns:
-            ignore_hdu_patterns = ' '.join(sorted(self.ignore_hdu_patterns))
-            self._writeln(' HDU(s) not to be compared:\n{}'
-                          .format(wrapper.fill(ignore_hdu_patterns)))
+            ignore_hdu_patterns = " ".join(sorted(self.ignore_hdu_patterns))
+            self._writeln(
+                " HDU(s) not to be compared:\n{}".format(
+                    wrapper.fill(ignore_hdu_patterns)
+                )
+            )
 
         if self.ignore_keywords:
-            ignore_keywords = ' '.join(sorted(self.ignore_keywords))
-            self._writeln(' Keyword(s) not to be compared:\n{}'
-                          .format(wrapper.fill(ignore_keywords)))
+            ignore_keywords = " ".join(sorted(self.ignore_keywords))
+            self._writeln(
+                " Keyword(s) not to be compared:\n{}".format(
+                    wrapper.fill(ignore_keywords)
+                )
+            )
 
         if self.ignore_comments:
-            ignore_comments = ' '.join(sorted(self.ignore_comments))
-            self._writeln(' Keyword(s) whose comments are not to be compared'
-                          ':\n{}'.format(wrapper.fill(ignore_comments)))
+            ignore_comments = " ".join(sorted(self.ignore_comments))
+            self._writeln(
+                " Keyword(s) whose comments are not to be compared:\n{}".format(
+                    wrapper.fill(ignore_comments)
+                )
+            )
 
         if self.ignore_fields:
-            ignore_fields = ' '.join(sorted(self.ignore_fields))
-            self._writeln(' Table column(s) not to be compared:\n{}'
-                          .format(wrapper.fill(ignore_fields)))
+            ignore_fields = " ".join(sorted(self.ignore_fields))
+            self._writeln(
+                " Table column(s) not to be compared:\n{}".format(
+                    wrapper.fill(ignore_fields)
+                )
+            )
 
-        self._writeln(' Maximum number of different data values to be '
-                      'reported: {}'.format(self.numdiffs))
-        self._writeln(' Relative tolerance: {}, Absolute tolerance: {}'
-                      .format(self.rtol, self.atol))
+        self._writeln(
+            " Maximum number of different data values to be reported: {}".format(
+                self.numdiffs
+            )
+        )
+        self._writeln(
+            " Relative tolerance: {}, Absolute tolerance: {}".format(
+                self.rtol, self.atol
+            )
+        )
 
         if self.diff_hdu_count:
-            self._fileobj.write('\n')
-            self._writeln('Files contain different numbers of HDUs:')
-            self._writeln(f' a: {self.diff_hdu_count[0]}')
-            self._writeln(f' b: {self.diff_hdu_count[1]}')
+            self._fileobj.write("\n")
+            self._writeln("Files contain different numbers of HDUs:")
+            self._writeln(f" a: {self.diff_hdu_count[0]}")
+            self._writeln(f" b: {self.diff_hdu_count[1]}")
 
             if not self.diff_hdus:
-                self._writeln('No differences found between common HDUs.')
+                self._writeln("No differences found between common HDUs.")
                 return
         elif not self.diff_hdus:
-            self._fileobj.write('\n')
-            self._writeln('No differences found.')
+            self._fileobj.write("\n")
+            self._writeln("No differences found.")
             return
 
         for idx, hdu_diff, extname, extver in self.diff_hdus:
             # print out the extension heading
             if idx == 0:
-                self._fileobj.write('\n')
-                self._writeln('Primary HDU:')
+                self._fileobj.write("\n")
+                self._writeln("Primary HDU:")
             else:
-                self._fileobj.write('\n')
+                self._fileobj.write("\n")
                 if extname:
-                    self._writeln(f'Extension HDU {idx} ({extname}, {extver}):')
+                    self._writeln(f"Extension HDU {idx} ({extname}, {extver}):")
                 else:
-                    self._writeln(f'Extension HDU {idx}:')
+                    self._writeln(f"Extension HDU {idx}:")
             hdu_diff.report(self._fileobj, indent=self._indent + 1)
 
 
@@ -448,9 +514,19 @@ class HDUDiff(_BaseDiff):
       HDUs containing non-empty data of an indeterminate type).
     """
 
-    def __init__(self, a, b, ignore_keywords=[], ignore_comments=[],
-                 ignore_fields=[], numdiffs=10, rtol=0.0, atol=0.0,
-                 ignore_blanks=True, ignore_blank_cards=True):
+    def __init__(
+        self,
+        a,
+        b,
+        ignore_keywords=[],
+        ignore_comments=[],
+        ignore_fields=[],
+        numdiffs=10,
+        rtol=0.0,
+        atol=0.0,
+        ignore_blanks=True,
+        ignore_blank_cards=True,
+    ):
         """
         Parameters
         ----------
@@ -539,28 +615,28 @@ class HDUDiff(_BaseDiff):
         if self.a.level != self.b.level:
             self.diff_extlevels = (self.a.level, self.b.level)
 
-        if self.a.header.get('XTENSION') != self.b.header.get('XTENSION'):
-            self.diff_extension_types = (self.a.header.get('XTENSION'),
-                                         self.b.header.get('XTENSION'))
+        if self.a.header.get("XTENSION") != self.b.header.get("XTENSION"):
+            self.diff_extension_types = (
+                self.a.header.get("XTENSION"),
+                self.b.header.get("XTENSION"),
+            )
 
-        self.diff_headers = HeaderDiff.fromdiff(self, self.a.header.copy(),
-                                                self.b.header.copy())
+        self.diff_headers = HeaderDiff.fromdiff(
+            self, self.a.header.copy(), self.b.header.copy()
+        )
 
         if self.a.data is None or self.b.data is None:
             # TODO: Perhaps have some means of marking this case
             pass
         elif self.a.is_image and self.b.is_image:
-            self.diff_data = ImageDataDiff.fromdiff(self, self.a.data,
-                                                    self.b.data)
+            self.diff_data = ImageDataDiff.fromdiff(self, self.a.data, self.b.data)
             # Clean up references to (possibly) memmapped arrays so they can
             # be closed by .close()
             self.diff_data.a = None
             self.diff_data.b = None
-        elif (isinstance(self.a, _TableLikeHDU) and
-              isinstance(self.b, _TableLikeHDU)):
+        elif isinstance(self.a, _TableLikeHDU) and isinstance(self.b, _TableLikeHDU):
             # TODO: Replace this if/when _BaseHDU grows a .is_table property
-            self.diff_data = TableDataDiff.fromdiff(self, self.a.data,
-                                                    self.b.data)
+            self.diff_data = TableDataDiff.fromdiff(self, self.a.data, self.b.data)
             # Clean up references to (possibly) memmapped arrays so they can
             # be closed by .close()
             self.diff_data.a = None
@@ -568,8 +644,7 @@ class HDUDiff(_BaseDiff):
         elif not self.diff_extension_types:
             # Don't diff the data for unequal extension types that are not
             # recognized image or table types
-            self.diff_data = RawDataDiff.fromdiff(self, self.a.data,
-                                                  self.b.data)
+            self.diff_data = RawDataDiff.fromdiff(self, self.a.data, self.b.data)
             # Clean up references to (possibly) memmapped arrays so they can
             # be closed by .close()
             self.diff_data.a = None
@@ -579,26 +654,36 @@ class HDUDiff(_BaseDiff):
         if self.identical:
             self._writeln(" No differences found.")
         if self.diff_extension_types:
-            self._writeln(" Extension types differ:\n  a: {}\n  "
-                          "b: {}".format(*self.diff_extension_types))
+            self._writeln(
+                " Extension types differ:\n  a: {}\n  b: {}".format(
+                    *self.diff_extension_types
+                )
+            )
         if self.diff_extnames:
-            self._writeln(" Extension names differ:\n  a: {}\n  "
-                          "b: {}".format(*self.diff_extnames))
+            self._writeln(
+                " Extension names differ:\n  a: {}\n  b: {}".format(*self.diff_extnames)
+            )
         if self.diff_extvers:
-            self._writeln(" Extension versions differ:\n  a: {}\n  "
-                          "b: {}".format(*self.diff_extvers))
+            self._writeln(
+                " Extension versions differ:\n  a: {}\n  b: {}".format(
+                    *self.diff_extvers
+                )
+            )
 
         if self.diff_extlevels:
-            self._writeln(" Extension levels differ:\n  a: {}\n  "
-                          "b: {}".format(*self.diff_extlevels))
+            self._writeln(
+                " Extension levels differ:\n  a: {}\n  b: {}".format(
+                    *self.diff_extlevels
+                )
+            )
 
         if not self.diff_headers.identical:
-            self._fileobj.write('\n')
+            self._fileobj.write("\n")
             self._writeln(" Headers contain differences:")
             self.diff_headers.report(self._fileobj, indent=self._indent + 1)
 
         if self.diff_data is not None and not self.diff_data.identical:
-            self._fileobj.write('\n')
+            self._fileobj.write("\n")
             self._writeln(" Data contains differences:")
             self.diff_data.report(self._fileobj, indent=self._indent + 1)
 
@@ -646,8 +731,17 @@ class HeaderDiff(_BaseDiff):
     all keywords that appear in both headers.
     """
 
-    def __init__(self, a, b, ignore_keywords=[], ignore_comments=[],
-                 rtol=0.0, atol=0.0, ignore_blanks=True, ignore_blank_cards=True):
+    def __init__(
+        self,
+        a,
+        b,
+        ignore_keywords=[],
+        ignore_comments=[],
+        rtol=0.0,
+        atol=0.0,
+        ignore_blanks=True,
+        ignore_blank_cards=True,
+    ):
         """
         Parameters
         ----------
@@ -715,12 +809,12 @@ class HeaderDiff(_BaseDiff):
         self.ignore_comment_patterns = set()
         for keyword in list(self.ignore_keywords):
             keyword = keyword.upper()
-            if keyword != '*' and glob.has_magic(keyword):
+            if keyword != "*" and glob.has_magic(keyword):
                 self.ignore_keywords.remove(keyword)
                 self.ignore_keyword_patterns.add(keyword)
         for keyword in list(self.ignore_comments):
             keyword = keyword.upper()
-            if keyword != '*' and glob.has_magic(keyword):
+            if keyword != "*" and glob.has_magic(keyword):
                 self.ignore_comments.remove(keyword)
                 self.ignore_comment_patterns.add(keyword)
 
@@ -757,8 +851,10 @@ class HeaderDiff(_BaseDiff):
             b = Header.fromstring(b)
 
         if not (isinstance(a, Header) and isinstance(b, Header)):
-            raise TypeError('HeaderDiff can only diff astropy.io.fits.Header '
-                            'objects or strings containing FITS headers.')
+            raise TypeError(
+                "HeaderDiff can only diff astropy.io.fits.Header "
+                "objects or strings containing FITS headers."
+            )
 
         super().__init__(a, b)
 
@@ -802,12 +898,10 @@ class HeaderDiff(_BaseDiff):
         keywordsb = keywordsb.difference(self.ignore_keywords)
         if self.ignore_keyword_patterns:
             for pattern in self.ignore_keyword_patterns:
-                keywordsa = keywordsa.difference(fnmatch.filter(keywordsa,
-                                                                pattern))
-                keywordsb = keywordsb.difference(fnmatch.filter(keywordsb,
-                                                                pattern))
+                keywordsa = keywordsa.difference(fnmatch.filter(keywordsa, pattern))
+                keywordsb = keywordsb.difference(fnmatch.filter(keywordsb, pattern))
 
-        if '*' in self.ignore_keywords:
+        if "*" in self.ignore_keywords:
             # Any other differences between keywords are to be ignored
             return
 
@@ -849,7 +943,7 @@ class HeaderDiff(_BaseDiff):
                 # No differences found; delete the array of Nones
                 del self.diff_keyword_values[keyword]
 
-            if '*' in self.ignore_comments or keyword in self.ignore_comments:
+            if "*" in self.ignore_comments or keyword in self.ignore_comments:
                 continue
             if self.ignore_comment_patterns:
                 skip = False
@@ -871,37 +965,47 @@ class HeaderDiff(_BaseDiff):
 
     def _report(self):
         if self.diff_keyword_count:
-            self._writeln(' Headers have different number of cards:')
-            self._writeln(f'  a: {self.diff_keyword_count[0]}')
-            self._writeln(f'  b: {self.diff_keyword_count[1]}')
+            self._writeln(" Headers have different number of cards:")
+            self._writeln(f"  a: {self.diff_keyword_count[0]}")
+            self._writeln(f"  b: {self.diff_keyword_count[1]}")
         if self.diff_keywords:
             for keyword in self.diff_keywords[0]:
                 if keyword in Card._commentary_keywords:
                     val = self.a[keyword][0]
                 else:
                     val = self.a[keyword]
-                self._writeln(f' Extra keyword {keyword!r:8} in a: {val!r}')
+                self._writeln(f" Extra keyword {keyword!r:8} in a: {val!r}")
             for keyword in self.diff_keywords[1]:
                 if keyword in Card._commentary_keywords:
                     val = self.b[keyword][0]
                 else:
                     val = self.b[keyword]
-                self._writeln(f' Extra keyword {keyword!r:8} in b: {val!r}')
+                self._writeln(f" Extra keyword {keyword!r:8} in b: {val!r}")
 
         if self.diff_duplicate_keywords:
             for keyword, count in sorted(self.diff_duplicate_keywords.items()):
-                self._writeln(f' Inconsistent duplicates of keyword {keyword!r:8}:')
-                self._writeln('  Occurs {} time(s) in a, {} times in (b)'
-                              .format(*count))
+                self._writeln(f" Inconsistent duplicates of keyword {keyword!r:8}:")
+                self._writeln(
+                    "  Occurs {} time(s) in a, {} times in (b)".format(*count)
+                )
 
         if self.diff_keyword_values or self.diff_keyword_comments:
             for keyword in self.common_keywords:
-                report_diff_keyword_attr(self._fileobj, 'values',
-                                         self.diff_keyword_values, keyword,
-                                         ind=self._indent)
-                report_diff_keyword_attr(self._fileobj, 'comments',
-                                         self.diff_keyword_comments, keyword,
-                                         ind=self._indent)
+                report_diff_keyword_attr(
+                    self._fileobj,
+                    "values",
+                    self.diff_keyword_values,
+                    keyword,
+                    ind=self._indent,
+                )
+                report_diff_keyword_attr(
+                    self._fileobj,
+                    "comments",
+                    self.diff_keyword_comments,
+                    keyword,
+                    ind=self._indent,
+                )
+
 
 # TODO: It might be good if there was also a threshold option for percentage of
 # different pixels: For example ignore if only 1% of the pixels are different
@@ -1004,8 +1108,10 @@ class ImageDataDiff(_BaseDiff):
         # Find the indices where the values are not equal
         # If neither a nor b are floating point (or complex), ignore rtol and
         # atol
-        if not (np.issubdtype(self.a.dtype, np.inexact) or
-                np.issubdtype(self.b.dtype, np.inexact)):
+        if not (
+            np.issubdtype(self.a.dtype, np.inexact)
+            or np.issubdtype(self.b.dtype, np.inexact)
+        ):
             rtol = 0
             atol = 0
         else:
@@ -1025,23 +1131,23 @@ class ImageDataDiff(_BaseDiff):
         else:
             numdiffs = self.numdiffs
 
-        self.diff_pixels = [(idx, (self.a[idx], self.b[idx]))
-                            for idx in islice(zip(*diffs), 0, numdiffs)]
+        self.diff_pixels = [
+            (idx, (self.a[idx], self.b[idx]))
+            for idx in islice(zip(*diffs), 0, numdiffs)
+        ]
         self.diff_ratio = float(self.diff_total) / float(len(self.a.flat))
 
     def _report(self):
         if self.diff_dimensions:
-            dimsa = ' x '.join(str(d) for d in
-                               reversed(self.diff_dimensions[0]))
-            dimsb = ' x '.join(str(d) for d in
-                               reversed(self.diff_dimensions[1]))
-            self._writeln(' Data dimensions differ:')
-            self._writeln(f'  a: {dimsa}')
-            self._writeln(f'  b: {dimsb}')
+            dimsa = " x ".join(str(d) for d in reversed(self.diff_dimensions[0]))
+            dimsb = " x ".join(str(d) for d in reversed(self.diff_dimensions[1]))
+            self._writeln(" Data dimensions differ:")
+            self._writeln(f"  a: {dimsa}")
+            self._writeln(f"  b: {dimsb}")
             # For now we don't do any further comparison if the dimensions
             # differ; though in the future it might be nice to be able to
             # compare at least where the images intersect
-            self._writeln(' No further data comparison performed.')
+            self._writeln(" No further data comparison performed.")
             return
 
         if not self.diff_pixels:
@@ -1049,14 +1155,23 @@ class ImageDataDiff(_BaseDiff):
 
         for index, values in self.diff_pixels:
             index = [x + 1 for x in reversed(index)]
-            self._writeln(f' Data differs at {index}:')
-            report_diff_values(values[0], values[1], fileobj=self._fileobj,
-                               indent_width=self._indent + 1)
+            self._writeln(f" Data differs at {index}:")
+            report_diff_values(
+                values[0],
+                values[1],
+                fileobj=self._fileobj,
+                indent_width=self._indent + 1,
+                rtol=self.rtol,
+                atol=self.atol,
+            )
 
         if self.diff_total > self.numdiffs:
-            self._writeln(' ...')
-        self._writeln(' {} different pixels found ({:.2%} different).'
-                      .format(self.diff_total, self.diff_ratio))
+            self._writeln(" ...")
+        self._writeln(
+            " {} different pixels found ({:.2%} different).".format(
+                self.diff_total, self.diff_ratio
+            )
+        )
 
 
 class RawDataDiff(ImageDataDiff):
@@ -1107,34 +1222,45 @@ class RawDataDiff(ImageDataDiff):
     def _diff(self):
         super()._diff()
         if self.diff_dimensions:
-            self.diff_dimensions = (self.diff_dimensions[0][0],
-                                    self.diff_dimensions[1][0])
+            self.diff_dimensions = (
+                self.diff_dimensions[0][0],
+                self.diff_dimensions[1][0],
+            )
 
         self.diff_bytes = [(x[0], y) for x, y in self.diff_pixels]
         del self.diff_pixels
 
     def _report(self):
         if self.diff_dimensions:
-            self._writeln(' Data sizes differ:')
-            self._writeln(f'  a: {self.diff_dimensions[0]} bytes')
-            self._writeln(f'  b: {self.diff_dimensions[1]} bytes')
+            self._writeln(" Data sizes differ:")
+            self._writeln(f"  a: {self.diff_dimensions[0]} bytes")
+            self._writeln(f"  b: {self.diff_dimensions[1]} bytes")
             # For now we don't do any further comparison if the dimensions
             # differ; though in the future it might be nice to be able to
             # compare at least where the images intersect
-            self._writeln(' No further data comparison performed.')
+            self._writeln(" No further data comparison performed.")
             return
 
         if not self.diff_bytes:
             return
 
         for index, values in self.diff_bytes:
-            self._writeln(f' Data differs at byte {index}:')
-            report_diff_values(values[0], values[1], fileobj=self._fileobj,
-                               indent_width=self._indent + 1)
+            self._writeln(f" Data differs at byte {index}:")
+            report_diff_values(
+                values[0],
+                values[1],
+                fileobj=self._fileobj,
+                indent_width=self._indent + 1,
+                rtol=self.rtol,
+                atol=self.atol,
+            )
 
-        self._writeln(' ...')
-        self._writeln(' {} different bytes found ({:.2%} different).'
-                      .format(self.diff_total, self.diff_ratio))
+        self._writeln(" ...")
+        self._writeln(
+            " {} different bytes found ({:.2%} different).".format(
+                self.diff_total, self.diff_ratio
+            )
+        )
 
 
 class TableDataDiff(_BaseDiff):
@@ -1269,7 +1395,7 @@ class TableDataDiff(_BaseDiff):
         colsa = {c.name.lower(): c for c in colsa}
         colsb = {c.name.lower(): c for c in colsb}
 
-        if '*' in self.ignore_fields:
+        if "*" in self.ignore_fields:
             # If all columns are to be ignored, ignore any further differences
             # between the columns
             return
@@ -1289,16 +1415,18 @@ class TableDataDiff(_BaseDiff):
 
         colsa_set = set(colsa.values())
         colsb_set = set(colsb.values())
-        self.common_columns = sorted(colsa_set.intersection(colsb_set),
-                                     key=operator.attrgetter('name'))
+        self.common_columns = sorted(
+            colsa_set.intersection(colsb_set), key=operator.attrgetter("name")
+        )
 
-        self.common_column_names = {col.name.lower()
-                                    for col in self.common_columns}
+        self.common_column_names = {col.name.lower() for col in self.common_columns}
 
-        left_only_columns = {col.name.lower(): col
-                             for col in colsa_set.difference(colsb_set)}
-        right_only_columns = {col.name.lower(): col
-                              for col in colsb_set.difference(colsa_set)}
+        left_only_columns = {
+            col.name.lower(): col for col in colsa_set.difference(colsb_set)
+        }
+        right_only_columns = {
+            col.name.lower(): col for col in colsb_set.difference(colsa_set)
+        }
 
         if left_only_columns or right_only_columns:
             self.diff_columns = (left_only_columns, right_only_columns)
@@ -1351,21 +1479,26 @@ class TableDataDiff(_BaseDiff):
                 valb = getattr(colb, attr, None)
                 if diff_values(vala, valb):
                     self.diff_column_attributes.append(
-                        ((col.name.upper(), attr), (vala, valb)))
+                        ((col.name.upper(), attr), (vala, valb))
+                    )
 
             arra = self.a[col.name]
             arrb = self.b[col.name]
 
-            if (np.issubdtype(arra.dtype, np.floating) and
-                    np.issubdtype(arrb.dtype, np.floating)):
-                diffs = where_not_allclose(arra, arrb,
-                                           rtol=self.rtol,
-                                           atol=self.atol)
-            elif 'P' in col.format:
-                diffs = ([idx for idx in range(len(arra))
-                          if not np.allclose(arra[idx], arrb[idx],
-                                             rtol=self.rtol,
-                                             atol=self.atol)],)
+            if np.issubdtype(arra.dtype, np.floating) and np.issubdtype(
+                arrb.dtype, np.floating
+            ):
+                diffs = where_not_allclose(arra, arrb, rtol=self.rtol, atol=self.atol)
+            elif "P" in col.format or "Q" in col.format:
+                diffs = (
+                    [
+                        idx
+                        for idx in range(len(arra))
+                        if not np.allclose(
+                            arra[idx], arrb[idx], rtol=self.rtol, atol=self.atol
+                        )
+                    ],
+                )
             else:
                 diffs = np.where(arra != arrb)
 
@@ -1389,41 +1522,46 @@ class TableDataDiff(_BaseDiff):
                     # interested in storing row-by-row differences
                     continue
                 last_seen_idx = idx
-                self.diff_values.append(((col.name, idx),
-                                         (arra[idx], arrb[idx])))
+                self.diff_values.append(((col.name, idx), (arra[idx], arrb[idx])))
 
         total_values = len(self.a) * len(self.a.dtype.fields)
         self.diff_ratio = float(self.diff_total) / float(total_values)
 
     def _report(self):
         if self.diff_column_count:
-            self._writeln(' Tables have different number of columns:')
-            self._writeln(f'  a: {self.diff_column_count[0]}')
-            self._writeln(f'  b: {self.diff_column_count[1]}')
+            self._writeln(" Tables have different number of columns:")
+            self._writeln(f"  a: {self.diff_column_count[0]}")
+            self._writeln(f"  b: {self.diff_column_count[1]}")
 
         if self.diff_column_names:
             # Show columns with names unique to either table
             for name in self.diff_column_names[0]:
                 format = self.diff_columns[0][name.lower()].format
-                self._writeln(f' Extra column {name} of format {format} in a')
+                self._writeln(f" Extra column {name} of format {format} in a")
             for name in self.diff_column_names[1]:
                 format = self.diff_columns[1][name.lower()].format
-                self._writeln(f' Extra column {name} of format {format} in b')
+                self._writeln(f" Extra column {name} of format {format} in b")
 
         col_attrs = dict(_COL_ATTRS)
         # Now go through each table again and show columns with common
         # names but other property differences...
         for col_attr, vals in self.diff_column_attributes:
             name, attr = col_attr
-            self._writeln(f' Column {name} has different {col_attrs[attr]}:')
-            report_diff_values(vals[0], vals[1], fileobj=self._fileobj,
-                               indent_width=self._indent + 1)
+            self._writeln(f" Column {name} has different {col_attrs[attr]}:")
+            report_diff_values(
+                vals[0],
+                vals[1],
+                fileobj=self._fileobj,
+                indent_width=self._indent + 1,
+                rtol=self.rtol,
+                atol=self.atol,
+            )
 
         if self.diff_rows:
-            self._writeln(' Table rows differ:')
-            self._writeln(f'  a: {self.diff_rows[0]}')
-            self._writeln(f'  b: {self.diff_rows[1]}')
-            self._writeln(' No further data comparison performed.')
+            self._writeln(" Table rows differ:")
+            self._writeln(f"  a: {self.diff_rows[0]}")
+            self._writeln(f"  b: {self.diff_rows[1]}")
+            self._writeln(" No further data comparison performed.")
             return
 
         if not self.diff_values:
@@ -1431,20 +1569,31 @@ class TableDataDiff(_BaseDiff):
 
         # Finally, let's go through and report column data differences:
         for indx, values in self.diff_values:
-            self._writeln(' Column {} data differs in row {}:'.format(*indx))
-            report_diff_values(values[0], values[1], fileobj=self._fileobj,
-                               indent_width=self._indent + 1)
+            self._writeln(" Column {} data differs in row {}:".format(*indx))
+            report_diff_values(
+                values[0],
+                values[1],
+                fileobj=self._fileobj,
+                indent_width=self._indent + 1,
+                rtol=self.rtol,
+                atol=self.atol,
+            )
 
         if self.diff_values and self.numdiffs < self.diff_total:
-            self._writeln(' ...{} additional difference(s) found.'.format(
-                                self.diff_total - self.numdiffs))
+            self._writeln(
+                " ...{} additional difference(s) found.".format(
+                    self.diff_total - self.numdiffs
+                )
+            )
 
         if self.diff_total > self.numdiffs:
-            self._writeln(' ...')
+            self._writeln(" ...")
 
-        self._writeln(' {} different table data element(s) found '
-                      '({:.2%} different).'
-                      .format(self.diff_total, self.diff_ratio))
+        self._writeln(
+            " {} different table data element(s) found ({:.2%} different).".format(
+                self.diff_total, self.diff_ratio
+            )
+        )
 
 
 def report_diff_keyword_attr(fileobj, attr, diffs, keyword, ind=0):
@@ -1459,11 +1608,13 @@ def report_diff_keyword_attr(fileobj, attr, diffs, keyword, ind=0):
             if val is None:
                 continue
             if idx == 0:
-                dup = ''
+                dup = ""
             else:
-                dup = f'[{idx + 1}]'
+                dup = f"[{idx + 1}]"
             fileobj.write(
-                fixed_width_indent(' Keyword {:8}{} has different {}:\n'
-                                   .format(keyword, dup, attr), ind))
-            report_diff_values(val[0], val[1], fileobj=fileobj,
-                               indent_width=ind + 1)
+                fixed_width_indent(
+                    f" Keyword {keyword:8}{dup} has different {attr}:\n",
+                    ind,
+                )
+            )
+            report_diff_values(val[0], val[1], fileobj=fileobj, indent_width=ind + 1)
